package csci291.gds.dungeoncrawl;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.io.File;
import java.io.FileNotFoundException;
import java.util.ArrayList;
import java.util.Random;
import java.util.Scanner;

import csci291.gds.console.*;

/**
 * A simple dungeon crawling game.
 * @author RustyMohawk
 *
 */
public class DungeonCrawl {

	static Scanner in = new Scanner(System.in);
	static char[][] map;
	static int height;
	static int width;
	static int playerY;
	static int playerX;
	static boolean win;
	static ArrayList<Enemy> enemies;
	static String message = "";
	
	static ConsoleGUI console;
	
	static DCEvent[] events = {
		new DCEvent("You fell into a pit!", false),
		new DCEvent("You are attacked by rats...", true),
		new DCEvent("You found a gem of true power...", true),
		new DCEvent("You were eaten by a Grue!", false),
		new DCEvent("You were swallowed by a Gelatinous Cube!", false),
		new DCEvent("You narrowly avoid a pit...", true),
		new DCEvent("You were crushed by a falling ceiling!", false),
		new DCEvent("You avoid being crushed by a falling ceiling...", true),
		new DCEvent("You are engulfed in flame!", false),
		new DCEvent("You avoid a jet of searing flames...", true),
		new DCEvent("You find a deceased adventurer...", true),
		new DCEvent("You were slain by a Minotaur!", false),
		new DCEvent("You are attacked by bats...", true),
		new DCEvent("The Goddess Siryiana appears to you!", false),
		new DCEvent("You were consumed by the Void!", false),
		new DCEvent("You are blessed by Alor...", true),
		new DCEvent("You were poisoned by arrows from the wall!", false),
		new DCEvent("You were evicerated by a knife from the dark!", false),
		new DCEvent("You stepped on a spike trap!", false),
		new DCEvent("You find a legendary sword...", true),
		new DCEvent("You were possessed by a dark spirit!", false),
		new DCEvent("You rescue the princess...", true)
	};
	
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		DungeonCrawl dc = new DungeonCrawl();
		dc.go();
	}

	public boolean cont = true;
	
	private void go() {
		
		console = new ConsoleGUI("Dungeon Crawl");
		console.setKeyInputHandler(new KeyInputHandler());
		keepGoing();
	}
	
	private void keepGoing() {
		String conf = "y";
		while (conf.equals("y") || conf.equals("Y")) {
			
			Random mapRand = new Random();
			String mapNum = Integer.toString(mapRand.nextInt(6));
			while (mapNum.length() < 3) {
				mapNum = "0" + mapNum;
			}
			File mapFile = new File(mapNum + ".dcm");
			
			win = false;
			
			enemies = new ArrayList<Enemy>();
			
			try {
				Scanner mapIn = new Scanner(mapFile);
				int i = 0;
				height = mapIn.nextInt();
				width = mapIn.nextInt();
				playerY = mapIn.nextInt();
				playerX = mapIn.nextInt();
				map = new char[height][width];
				mapIn.nextLine();
				while (mapIn.hasNextLine()) {
					map[i] = mapIn.nextLine().toCharArray();
					for (int j = 0; j < map[i].length; j++) {
						if (map[i][j] == '&') {
							enemies.add(new Enemy(i, j));
							//console.printLine("Enemy: " + i + " " + j);
						}
					}
					i++;
				}
				
				boolean cont = true;
				while (cont) {
					for (int j = 0; j < map.length; j++) {
						console.print(new String(map[j]));
						console.printLine("");
					}
					int input = in.nextInt();
					switch(input) {
					case 1:
						cont = move(playerY - 1, playerX);
						break;
					case 2:
						cont = move(playerY, playerX + 1);
						break;
					case 3:
						cont = move(playerY, playerX - 1);
						break;
					case 4:
						cont = move(playerY + 1, playerX);
						break;
					}
					for (int k = 0; k < enemies.size(); k++) {
						if (enemies.get(k).move(map)) {
							win = false;
							cont = false;
							console.printLine(enemies.get(k).y + " " + enemies.get(k).x);
						}
					}
				}
				if (win) {
					console.printLine("You Win!");
				} else {
					console.printLine("You Lose...");
				}
			} catch (FileNotFoundException e) {
				e.printStackTrace();
			}
		}
	}

	/**
	 * 
	 * @param py2 The new y-location.
	 * @param px2 The new x-location.
	 * @return True to continue the game or false to quit.
	 */
	private static boolean move(int py2, int px2) {
		boolean cont = true;
		
		if (map[py2][px2] == '!') {
			Random r = new Random();
			int rand = r.nextInt(events.length);
			message += "\n" + events[rand].desc;
			cont = events[rand].cont;
		}
		
		if (map[py2][px2] == '*') {
			win = true;
			cont = false;
//		} else if (map[py2][px2] == '&') {
//			win = false;
//			cont = false;
		} else if (playerY != 0 && playerX != 0 && playerY != height
				&& playerX != width && map[py2][px2] != '#') {
			map[playerY][playerX] = ' ';
			playerY = py2;
			playerX = px2;
			map[playerY][playerX] = '@';
		}
		return cont;
	}

	private class KeyInputHandler extends KeyAdapter {
		
		public void keyReleased(KeyEvent e) {			
			if(cont) {
				if (e.getKeyCode() == KeyEvent.VK_UP) {
					cont  = move(playerY - 1, playerX);
				}
				if (e.getKeyCode() == KeyEvent.VK_DOWN) {
					cont = move(playerY + 1, playerX);
				}
				if (e.getKeyCode() == KeyEvent.VK_LEFT) {
					cont = move(playerY, playerX - 1);
				}
				if (e.getKeyCode() == KeyEvent.VK_RIGHT) {
					cont = move(playerY, playerX + 1);
				}
			
				console.clear();
				for (int j = 0; j < map.length; j++) {
					console.print(new String(map[j]));
					console.printLine("");
				}
				console.printLine(message);
				
				if(!cont) {
					if (win) {
						console.printLine("You escaped the dungeon!\nYou Win!");
					} else {
						console.printLine("You Lose...");
					}
				}
			}
		}
		
		public void keyTyped(KeyEvent e) {
			// if we hit escape, then quit the game
			if (e.getKeyChar() == 27) {
				console.exit();
			}
		}
	}

}
